#!/bin/bash

#
# Mirror source directory to destination directory
#
# note: prior to running this script:
# 1) SOURCE and DEST variables set to endpoints and paths to mirror.
# 2) Endpoints must be activated.
#    (Globus Connect endpoints and shared endpoints do not require activation)
# 3) The proper username must be encoded in the SSH variable.
# 4) To run from cron, set SSH_AUTH_SOCK to the value from your login shell
#

SOURCE_ENDPOINT="tuecke#share1"
SOURCE_PATH="/~/mydata/"

DEST_ENDPOINT="tuecke#mylaptop"
DEST_PATH="/~/tutorial/mydata/"

SOURCE="$SOURCE_ENDPOINT$SOURCE_PATH"
DEST="$DEST_ENDPOINT$DEST_PATH"

SSH="ssh tuecke@cli.globusonline.org v=transfer/2.5:mirror/tutorial"

#
# Hack to allow script to run from cron, by using SSH agent security
# that is being used by your login shell.
# To run properly from cron, you should create a non-password-protected
# SSH key for your cron jobs and add that to your Globus Online profile.
#
if [ -z "$SSH_AUTH_SOCK" ]; then
	SSH_AUTH_SOCK=/tmp/launch-BevJ9C/Listeners
	export SSH_AUTH_SOCK
fi

#
# Use a local lock file to sense whether there is another mirror operation
# already running
#
exit_code=0
mirror_lockfile="$HOME/.mirror_lockfile"
if [ -f "$mirror_lockfile" ]; then
	echo "There is already a mirror operation in progress"
	exit
fi
touch "$mirror_lockfile"

cleanup_and_exit () {
	rm -f $mirror_lockfile
	rm -f /tmp/mirror.faults.$task_id
	exit $exit_code
}

#
# Function: Check return code from a Globus Online command
#
check_rc () {
	if [ $rc -ne 0 ]; then
		if [ $rc -eq 255 ]; then
			echo "ERROR: ssh failed to connect - aborting."
			exit_code=$rc
			cleanup_and_exit
		else
			echo "ERROR: Globus Online transfer command failed"
			exit_code=$rc
			cleanup_and_exit
		fi
	fi
}

#
# Check if endpoints are activated
#
status=$($SSH endpoint-list -f credential_status $SOURCE_ENDPOINT)
rc=$?
check_rc
if [ $status != "ACTIVE" ]; then
	echo "ERROR: Source endpoint not activated: $SOURCE_ENDPOINT"
	exit_code=1
	cleanup_and_exit
fi

status=$($SSH endpoint-list -f credential_status $DEST_ENDPOINT)
rc=$?
check_rc
if [ $status != "ACTIVE" ]; then
	echo "ERROR: Destination endpoint not activated: $DEST_ENDPOINT"
	exit_code=1
	cleanup_and_exit
fi

#
# Obtain a Globus Online transfer task id
#
echo "Get task id"
task_id=$($SSH transfer --generate-id)
rc=$?
check_rc
echo "Task id=$task_id"

#
# Submit the transfer task
#
TRANSFER_INPUT="$SOURCE $DEST -r"
echo "Submitting transfer"
$SSH transfer --taskid=$task_id -s 3 --label="mirror-tutorial" --preserve-mtime --delete --verify-checksum -- "$TRANSFER_INPUT"
rc=$?
check_rc

#
# loop checking for completion and for errors
#
tries=1
faults=0
while true; do
	status=$($SSH status $task_id -f status -O csv | tr -d '\r\n')
	echo "Status: $status"

	if [ $status == "SUCCEEDED" ]; then
		echo "Succeeded"
		break
	elif [ $status == "FAILED" ]; then
		echo "Failed"
		break
	else
		new_faults=$($SSH details $task_id -f faults -O csv | tr -d '\r\n')
		if [ $new_faults -gt $faults ]; then
			faults=$new_faults
			rm -f /tmp/mirror.faults.$task_id
			$SSH events $task_id --faults -f code -O csv | sort | uniq > /tmp/mirror.faults.$task_id
			uniq_faults=$(wc -l < /tmp/mirror.faults.$task_id)
			if [ $uniq_faults -gt 0 ]; then
   		 		if [ $(grep PERMISSION_DENIED /tmp/mirror.faults.$task_id | wc -l) -ne 0 ]; then
					echo "FAULT: Permission denied - cancel and exit"
					fatal_error=1
   		 		elif [ $(grep NO_SPACE_LEFT /tmp/mirror.faults.$task_id | wc -l) -ne 0 ]; then
					echo "FAULT: No space left on device - cancel and exit"
					fatal_error=1
   		 		elif [ $(grep QUOTA_EXCEEDED /tmp/mirror.faults.$task_id | wc -l) -ne 0 ]; then
					echo "FAULT: Quota exceeded - cancel and exit"
					fatal_error=1
				else
					echo "non-fatal faults: $faults"
					cat /tmp/mirror.faults.$task_id
					fatal_error=0
				fi
			fi
			if [ $fatal_error -ne 0 ]; then
				$SSH cancel $task_id
				exit_code=1
				cleanup_and_exit
			fi
		fi
	fi

	sleep 5

	if [ $tries -ge 16 ]; then
		echo "Giving up after $tries tries"
		$SSH cancel $task_id
		exit_code=1
		cleanup_and_exit
	else	
		let "tries+=1"
	fi
done

#
# Print out final job details
#
$SSH details $task_id

cleanup_and_exit

